`timescale 1ns / 1ns
`default_nettype none

// come on
module top(
	input wire nRST,
	input wire CLK,
	input wire SET_MIN_PLUS,
	input wire SET_MIN_MINUS,
	input wire SET_SEC_PLUS,
	input wire SET_SEC_MINUS,
	input wire COUNT_EN,
	output wire [7:0] OUTP0,
	output wire [7:0] OUTP1,
	output wire COMMON0,
	output wire COMMON1,
	output wire COMMON2,
	output wire COMMON3,
	output wire BUZZER
);

	wire clk_100hz, clk_0_66hz;
	// Clock Divider {{{
	clk_divider u_div_100hz(
		.nRST(nRST),
		.CLK_IN(CLK),
		.COUNTER_MAX(32'd624_999),
		.CLK_OUT(clk_100hz)
	);
	clk_divider u_div_0_66hz(
		.nRST(nRST),
		.CLK_IN(CLK),
		.COUNTER_MAX(32'd41_666_249),
		.CLK_OUT(clk_0_66hz)
	);
	/// }}}
	wire pulse_1hz;
	pulsegen_1hz u_pulse1hz(
		.nRST(nRST),
		.CLK(CLK),
		.OUTP(pulse_1hz)
	);

	wire chat_set_min_plus_lo, chat_set_min_plus_hi;
	wire chat_set_min_minus_lo, chat_set_min_minus_hi;
	wire chat_set_sec_plus_lo, chat_set_sec_plus_hi;
	wire chat_set_sec_minus_lo, chat_set_sec_minus_hi;
	// Anti Chattering {{{
	anti_chattering u_chat_minplus(
		.nRST(nRST),
		.CLK(clk_100hz),
		.INP(SET_MIN_PLUS),
		.OUTP_HI(chat_set_min_plus_hi),
		.OUTP_LO(chat_set_min_plus_lo)
	);
	anti_chattering u_chat_minminus(
		.nRST(nRST),
		.CLK(clk_100hz),
		.INP(SET_MIN_MINUS),
		.OUTP_HI(chat_set_min_minus_hi),
		.OUTP_LO(chat_set_min_minus_lo)
	);
	anti_chattering u_chat_secplus(
		.nRST(nRST),
		.CLK(clk_100hz),
		.INP(SET_SEC_PLUS),
		.OUTP_HI(chat_set_sec_plus_hi),
		.OUTP_LO(chat_set_sec_plus_lo)
	);
	anti_chattering u_chat_secminus(
		.nRST(nRST),
		.CLK(clk_100hz),
		.INP(SET_SEC_MINUS),
		.OUTP_HI(chat_set_sec_minus_hi),
		.OUTP_LO(chat_set_sec_minus_lo)
	);
	// }}}

	wire edge_set_min_plus, edge_set_min_minus;
	wire edge_set_sec_plus, edge_set_sec_minus;
	// Posedge Trigger {{{
	posedge_trigger u_edge_setminplus(
		.nRST(nRST),
		.CLK(CLK),
		.INP(chat_set_min_plus_hi),
		.OUTP(edge_set_min_plus)
	);
	posedge_trigger u_edge_setminminus(
		.nRST(nRST),
		.CLK(CLK),
		.INP(chat_set_min_minus_hi),
		.OUTP(edge_set_min_minus)
	);
	posedge_trigger u_edge_setsecplus(
		.nRST(nRST),
		.CLK(CLK),
		.INP(chat_set_sec_plus_hi),
		.OUTP(edge_set_sec_plus)
	);
	posedge_trigger u_edge_setsecminus(
		.nRST(nRST),
		.CLK(CLK),
		.INP(chat_set_sec_minus_hi),
		.OUTP(edge_set_sec_minus)
	);
	// }}}

	wire [3:0] number_1, number_100;
	wire [3:0] number_10, number_1000;
	assign number_10[3] = 0;
	assign number_1000[3] = 0;

	wire all_cnt_is_zero = {number_1000, number_100, number_10, number_1} == 16'd0;
	wire cnt12_full, cnt34_full;
	// Counter {{{
	counter60_bcd_down_with_en u_cnt60dn_12(
		.nRST(nRST),
		.CLK(CLK),
		.EN(pulse_1hz & COUNT_EN & ~all_cnt_is_zero),
		.INCR(edge_set_sec_plus),
		.DECR(edge_set_sec_minus),
		.NUMBER_1(number_1),
		.NUMBER_10(number_10[2:0]),
		.FULL(cnt12_full)
	);
	counter60_bcd_down_with_en u_cnt60dn_34(
		.nRST(nRST),
		.CLK(CLK),
		.EN(pulse_1hz & cnt12_full & COUNT_EN & ~all_cnt_is_zero),
		.INCR(edge_set_min_plus),
		.DECR(edge_set_min_minus),
		.NUMBER_1(number_100),
		.NUMBER_10(number_1000[2:0]),
		.FULL(cnt34_full)
	);
	// }}}

	buzzer_controller u_buzctrl(
		.nRST(nRST),
		.CLK(CLK),
		.CLK_DIVISOR(32'd62_499), // 1000Hz
		.OUTPUT_EN( all_cnt_is_zero & COUNT_EN ),
		.OUTP(BUZZER)
	);

	assign OUTP0[7] = 1;
	assign OUTP1[7] = 1;
	// Seg Decoder {{{
	seg_decoder_2digits u_seg01(
		.nRST(nRST),
		.CLK(clk_100hz),
		.NUMBER0(number_1),
		.NUMBER1(number_10),
		.COMMON0(COMMON0),
		.COMMON1(COMMON1),
		.IS_ANODE_COMMON(1'd1),
		.OUTPUT(OUTP0[6:0])
	);
	seg_decoder_2digits u_seg23(
		.nRST(nRST),
		.CLK(clk_100hz),
		.NUMBER0(number_100),
		.NUMBER1(number_1000),
		.COMMON0(COMMON2),
		.COMMON1(COMMON3),
		.IS_ANODE_COMMON(1'd1),
		.OUTPUT(OUTP1[6:0])
	);
	// }}}

endmodule

// TODO: Count Enable Slide SW
//       Buzzer
